#include <PalmOS.h>
#include "mymath.h"
#include "physics.h"

ball_t MoveAndDecelerate(ball_t ball, float t) {
    /* Move the ball one step along its path, and also slow the ball
       due to friction. Takes in a ball structure and a length of time
       in seconds for arguments. Returns a new ball structure. */
    float delta, speed;
    ball.position = AddVectors(ball.position,
      ScalarMultiplyVector(t, ball.velocity));
    speed = MagnitudeVector(ball.velocity);
    delta = FRICTION_DECELERATION * t;
    if (delta > speed)
        ball.velocity = ScalarMultiplyVector(0.0, ball.velocity);
    else {
        float fraction = (speed - delta) / speed;
        ball.velocity = ScalarMultiplyVector(fraction, ball.velocity);
    }
    return ball;
}

Boolean DetectCollision(ball_t moving, ball_t stationary) {
    /* Determines whether the first ball is colliding with the second ball. */
    vector_t position_offset;
    position_offset = SubtractVectors(stationary.position, moving.position);
    if (MagnitudeVector(position_offset) > 2 * BALL_RADIUS)
        return false;
    if (DotProductVectors(moving.velocity, position_offset) > 0)
        return true;
    else
        return false;
}

vector_t Collide(ball_t moving, ball_t stationary) {
    /* Returns a vector containing the velocity to transfer from the
     * moving ball to the stationary ball. */
    return ProjectVector(moving.velocity,
      SubtractVectors(stationary.position, moving.position));
}

vector_t Strike(vector_t start_spot, vector_t end_spot,
                vector_t transfer_velocity, vector_t ball_center) {
    /* For arguments this function takes a path of the cue stick and
     * the velocity we would want to add to the ball if we make a direct hit.
     * Returns the net velocity to be added to the target ball, or the zero
     * vector if we miss the ball. */
    float u, adjacent_length;
    vector_t closest_point, contact_point;
    vector_t start_to_end, closest_to_center, adjacent, start_to_contact;
    const vector_t zero_velocity = { 0.0, 0.0 };
    if (MagnitudeVector(SubtractVectors(ball_center, start_spot)) < BALL_RADIUS)
        return zero_velocity;
    start_to_end = SubtractVectors(end_spot, start_spot);
    u = ((ball_center.x - start_spot.x) * (end_spot.x - start_spot.x) +
         (ball_center.y - start_spot.y) * (end_spot.y - start_spot.y)) /
        DotProductVectors(start_to_end, start_to_end);
    closest_point = AddVectors(start_spot, ScalarMultiplyVector(u,
      SubtractVectors(end_spot, start_spot)));
    closest_to_center = SubtractVectors(ball_center, closest_point);
    if (MagnitudeVector(closest_to_center) >= BALL_RADIUS)
        return zero_velocity;
    adjacent_length = MySqrt(BALL_RADIUS * BALL_RADIUS -
      DotProductVectors(closest_to_center, closest_to_center));
    adjacent = ScalarMultiplyVector(adjacent_length / MagnitudeVector(start_to_end),
      start_to_end);
    contact_point = SubtractVectors(closest_point, adjacent);
    start_to_contact = SubtractVectors(contact_point, start_spot);
    if (DotProductVectors(start_to_end, SubtractVectors(contact_point, start_spot)) <
          DotProductVectors(start_to_contact, start_to_contact))
        return zero_velocity;
    return ProjectVector(transfer_velocity, SubtractVectors(ball_center, contact_point));
}
   
ball_t BounceWall(ball_t ball) {
    /* Bounce our ball off any of the four walls. Returns the ball structure
     * with a possibly modified velocity value. */
    extern RectangleType displayrect; /* defined in billiards.c */
    if ((ball.position.x < displayrect.topLeft.x + WALL_WIDTH + BALL_RADIUS &&
         ball.velocity.x < 0) ||
        (ball.position.x > displayrect.topLeft.x + displayrect.extent.x - WALL_WIDTH - BALL_RADIUS &&
         ball.velocity.x > 0))
        ball.velocity.x = -ball.velocity.x;
    if ((ball.position.y < displayrect.topLeft.y + WALL_WIDTH + BALL_RADIUS &&
         ball.velocity.y < 0) ||
        (ball.position.y > displayrect.topLeft.y + displayrect.extent.y - WALL_WIDTH - BALL_RADIUS &&
         ball.velocity.y > 0))
        ball.velocity.y = -ball.velocity.y;
    return ball;
}
